#!/usr/bin/python3
# Released under GPLv3+ License
# Danial Behzadi<dani.behzi@ubuntu.com>, 2019-2022.

"""
Main module for carburetor
"""

from sys import argv

import gi

gi.require_versions({"Adw": "1"})
from gi.repository import Adw

from . import actions
from . import ui


class Application(Adw.Application):
    """
    main window of carburetor
    """

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(
            *args, application_id="io.frama.tractor.carburetor", **kwargs
        )
        self.window = None
        self.prefs = None
        self.about = None

    def do_startup(self, *args, **kwargs) -> None:
        Adw.Application.do_startup(self)
        actions.do_startup(self)

    def do_activate(self, *args, **kwargs) -> None:
        if not self.window:
            ui.css()
            window = ui.get("MainWindow")
            self.add_window(window)
            self.window = window
        self.window.present()


def main() -> None:
    """
    main entrance of app
    """
    ui.initialize_builder()
    app = Application()
    app.run(argv)


if __name__ == "__main__":
    main()



#!/usr/bin/python3
# Released under GPLv3+ License
# Danial Behzadi <dani.behzi@ubuntu.com>, 2022.

"""
tasks to be done async
"""

import re

from subprocess import PIPE, Popen

from gi.repository import GLib

from . import config
from . import actions


def connect(action: str, app) -> None:
    """
    connect or disconnect
    """
    task = Popen(
        [config.COMMAND, action],
        stdout=PIPE,
        start_new_session=True,
    )
    if action == "start":
        config.dconf.set_int("pid", task.pid)
    app.io_in = GLib.io_add_watch(task.stdout, GLib.IO_IN, set_progress)
    GLib.io_add_watch(task.stdout, GLib.IO_HUP, thread_finished, app)


def set_progress(stdout, *_) -> bool:
    """
    set progress output on page desription
    """
    try:
        line = stdout.readline().decode("utf-8")
        valid = re.compile(r".*\[notice\] ")
        if "notice" in line:
            percentage = line.split(" ")[5]
            notice = valid.sub("", line)[:-9]
            actions.set_description(notice)
            actions.set_progress(int(percentage[:-1]))
    except ValueError:
        return False
    return True


def thread_finished(stdout, condition, app) -> bool:
    """
    things to do after process finished
    """
    if condition:
        GLib.source_remove(app.io_in)
        stdout.close()
        actions.set_run_status(app)
        return False
    return True


def new_id() -> None:
    """
    get new identity
    """
    with Popen([config.COMMAND, "newid"], stdout=PIPE) as task:
        task.wait()


def set_proxy(action: str) -> None:
    """
    set or unset proxy
    """
    with Popen([config.COMMAND, action], stdout=PIPE) as task:
        task.wait()


def get_bridges_file() -> str:
    """
    get bridges file
    """
    with Popen([config.COMMAND, "bridgesfile"], stdout=PIPE) as task:
        bridges = task.stdout.read()
        bridges_file = bridges.decode("utf-8").strip("\n")
        return bridges_file


def strtobool(val):
    """Convert a string representation of truth to true (1) or false (0).
    True values are 'y', 'yes', 't', 'true', 'on', and '1'; false values
    are 'n', 'no', 'f', 'false', 'off', and '0'.  Raises ValueError if
    'val' is anything else.
    """
    val = val.lower()
    if val in ("y", "yes", "t", "true", "on", "1"):
        return 1
    if val in ("n", "no", "f", "false", "off", "0"):
        return 0
    raise ValueError(f"invalid truth value {(val,)}")


def is_proxy_set() -> bool:
    """
    checks if proxy was already set
    """
    with Popen([config.COMMAND, "isset"], stdout=PIPE) as check:
        output = strtobool(check.stdout.read().decode("utf-8").strip("\n"))
        return output


def is_running() -> bool:
    """
    check if tractor is running or not
    """
    with Popen([config.COMMAND, "isrunning"], stdout=PIPE) as check:
        output = strtobool(check.stdout.read().decode("utf-8").strip("\n"))
        return output


def is_connected() -> bool:
    """
    check if tractor is connected or not
    """
    check = Popen([config.COMMAND, "isconnected"], stdout=PIPE)
    GLib.io_add_watch(check.stdout, GLib.IO_IN, return_connection_status)


def return_connection_status(stdout, *_) -> bool:
    """
    return connection status
    """
    result = stdout.readline().decode("utf-8")
    if "True" in result:
        toast = config._("Tractor is connected")
    else:
        toast = config._("Tractor couldn't connect")
    actions.notify(toast)
    return True



#!/usr/bin/python3
# Released under GPLv3+ License
# Danial Behzadi<dani.behzi@ubuntu.com>, 2020-2022.

"""
actions for carburetor
"""

import os
import re
import signal

import gi

gi.require_versions({"Adw": "1"})
from gi.repository import Gio, Adw

from . import config
from . import tasks
from . import ui


def add(name: str, function, app) -> None:
    """
    adds functions to app as actions
    """
    action = Gio.SimpleAction.new(name, None)
    action.connect("activate", function, app)
    app.add_action(action)


def do_startup(app) -> None:
    """
    actions to do when starting the app up
    """
    add("preferences", on_preferences, app)
    add("about", on_about, app)
    add("quit", on_quit, app)
    add("connect", on_connect, app)
    add("new_id", on_new_id, app)
    add("check_connection", on_check, app)
    add("toggle_proxy", on_toggle_proxy, app)
    add("cancel", on_cancel, app)
    add("save", on_save, app)


def on_preferences(*argv) -> None:
    """
    show the preferences window
    """
    app = argv[2]
    if not app.prefs:
        prefs_window = ui.get("PreferencesWindow")
        prefs_window.set_transient_for(app.window)
        app.prefs = prefs_window
    app.prefs.show()


def on_about(*argv) -> None:
    """
    show the about window
    """
    app = argv[2]
    if not app.about:
        about_dialog = ui.get("AboutDialog")
        about_dialog.set_transient_for(app.window)
        app.about = about_dialog
    app.about.show()


def on_quit(*argv) -> None:
    """
    exit the app
    """
    app = argv[2]
    app.quit()


def on_connect(*argv) -> None:
    """
    clicking on connect button
    """
    app = argv[2]
    button = ui.get("SplitButton")
    button.set_sensitive(False)
    progress_bar = ui.get("ProgressBar")
    set_progress(0)
    progress_bar.show()
    cancel_button = ui.get("CancelButton")
    cancel_button.set_visible(True)
    page = ui.get("MainPage")
    page.set_icon_name("image-loading-symbolic")
    if tasks.is_running():
        text_stopping = config._("Disconnecting…")
        page.set_title(text_stopping)
        action = "stop"
    else:
        text_starting = config._("Connecting…")
        page.set_title(text_starting)
        action = "start"
    tasks.connect(action, app)


def on_new_id(*_, **__) -> None:
    """
    clicking on new id button
    """
    if tasks.is_running():
        tasks.new_id()
        toast = config._("You have a new identity!")
    else:
        toast = config._("Tractor is not running!")
    notify(toast)


def on_check(*_, **__) -> None:
    """
    checks if tractor is connected or not
    """
    tasks.is_connected()


def on_toggle_proxy(*_, **__) -> None:
    """
    toggle proxy mode on system
    """
    if tasks.is_proxy_set():
        tasks.set_proxy("unset")
        toast = "Proxy has been unset"
    else:
        tasks.set_proxy("set")
        toast = "Proxy has been set"
    notify(toast)


def on_cancel(*_, **__) -> None:
    """
    abort the connection
    """
    dconf = config.dconf
    pid = dconf.get_int("pid")
    os.killpg(os.getpgid(pid), signal.SIGTERM)
    dconf.reset("pid")


def on_save(*_, **__) -> None:
    """
    clicking on save button in bridges
    """
    textview = ui.get("BridgesTextView")
    buff = textview.get_buffer()
    text = buff.get_text(buff.get_start_iter(), buff.get_end_iter(), 0)
    regex = re.compile(r"^( )*([Bb][Rr][Ii][Dd][Gg][Ee])?( )*", re.MULTILINE)
    bridges_file = tasks.get_bridges_file()
    if text == regex.sub("Bridge ", text):
        with open(bridges_file, "w", encoding="utf-8") as file:
            file.write(text)
    else:
        dialog = ui.get("BridgErrorDialog")
        dialog.show()


def set_description(text: str) -> None:
    """
    set description on main page
    """
    page = ui.get("MainPage")
    page.set_description(text)


def set_progress(percentage: int) -> None:
    """
    set progressbar percentage
    """
    progress_bar = ui.get("ProgressBar")
    fraction = float(percentage) / 100
    progress_bar.set_fraction(fraction)


def notify(text: str) -> None:
    """
    show toast
    """
    overlay = ui.get("ToastOverlay")
    toast = Adw.Toast()
    toast.set_title(text)
    overlay.add_toast(toast)


def set_to_stopped(app) -> None:
    """
    set status to stopped
    """
    page = ui.get("MainPage")
    page.set_icon_name("process-stop-symbolic")
    page.set_title(config._("Stopped"))
    page.set_description("")
    button = ui.get("SplitButton")
    text_start = config._("_Connect")
    style = button.get_style_context()
    style.remove_class("destructive-action")
    style.add_class("suggested-action")
    button.set_label(text_start)
    action_menu = button.get_popover()
    action_menu.set_sensitive(False)
    button = ui.get("CancelButton")
    button.set_visible(False)
    dconf = config.dconf
    dconf.reset("pid")
    if app:  # don't run on startup
        notify(config._("Tractor is stopped"))


def set_to_running(app) -> None:
    """
    set status to connected
    """
    page = ui.get("MainPage")
    page.set_icon_name("security-high-symbolic")
    page.set_title(config._("Running"))
    page.set_description("")
    button = ui.get("SplitButton")
    text_stop = config._("_Disconnect")
    style = button.get_style_context()
    style.remove_class("suggested-action")
    style.add_class("destructive-action")
    button.set_label(text_stop)
    action_menu = button.get_popover()
    action_menu.set_sensitive(True)
    button = ui.get("CancelButton")
    button.set_visible(False)
    if app:  # don't run on startup
        notify(config._("Tractor is running"))


def set_run_status(app=None) -> None:
    """
    set status of conection
    """
    if tasks.is_running():
        set_to_running(app)
    else:
        set_to_stopped(app)
    button = ui.get("SplitButton")
    button.set_sensitive(True)
    progress_bar = ui.get("ProgressBar")
    progress_bar.hide()


def set_pluginrow_sensivity(row=None) -> None:
    """
    set row sensitive if a plugable transport is set
    """
    if not row:
        row = ui.get("PluginRow")
    bridgetype = config.dconf.get_int("bridge-type")
    if bridgetype > 1:
        row.set_sensitive(True)
    else:
        row.set_sensitive(False)


def setup_pluginbutton(button=None) -> None:
    """
    set plugin button label and chooser
    """
    if not button:
        button = ui.get("PluginButton")
    dconf = config.dconf
    filename = Gio.File.new_for_path(dconf.get_string("plugable-transport"))
    if filename.query_exists():
        basename = filename.get_basename()
        button.set_label(basename)
    chooser = ui.get("PluginChooser")
    chooser.set_initial_file(filename)
    button.chooser = chooser



#!/usr/bin/python3
# Released under GPLv3+ License
# Danial Behzadi<dani.behzi@ubuntu.com>, 2020-2023.

"""
handlers for ui events
"""

from gi.repository import Gio, GLib, Gtk, GObject

from . import actions
from . import config
from . import tasks


class Country(GObject.Object):
    """
    prepare factory for countries
    """

    __gtype_name__ = "Country"

    def __init__(self, country_id: str, country_name: str):
        super().__init__()
        self._country_id = country_id
        self._country_name = country_name

    @GObject.Property
    def country_id(self) -> str:
        """
        return ids
        """
        return self._country_id

    @GObject.Property
    def country_name(self) -> str:
        """
        return names
        """
        return self._country_name


class BridgeType(GObject.Object):
    """
    Prepare factory for bridge types
    """

    __gtype_name__ = "BridgeType"

    def __init__(self, type_id: int, type_name: str):
        super().__init__()
        self._bridgetype_id = type_id
        self._bridgetype_name = type_name

    @GObject.Property
    def type_id(self) -> int:
        """
        return ids
        """
        return self._bridgetype_id

    @GObject.Property
    def type_name(self) -> str:
        """
        return names
        """
        return self._bridgetype_name


def on_mainpage_realize(window):
    """
    First setups
    """
    if window:
        actions.set_run_status()


def on_factory_setup(factory, list_item):
    """
    set labels for dropdown
    """
    del factory
    label = Gtk.Label()
    list_item.set_child(label)


def on_factory_bind(factory, list_item):
    """
    set country names as labels
    """
    del factory
    label = list_item.get_child()
    country = list_item.get_item()
    label.set_text(country.country_name)


def on_exitcountry_change(combo, _):
    """
    set exit country in dconf
    """
    country = combo.get_selected_item()
    node = country.country_id
    config.dconf.set_string("exit-node", node)


def on_exitcountry_realize(combo):
    """
    sets up exit node comborow
    """
    _ = config._
    nodes = {
        "au": _("Austria"),
        "bg": _("Bulgaria"),
        "ca": _("Canada"),
        "ch": _("Switzerland"),
        "cz": _("Czech"),
        "de": _("Germany"),
        "es": _("Spain"),
        "fi": _("Finland"),
        "fr": _("France"),
        "ie": _("Ireland"),
        "md": _("Moldova"),
        "nl": _("Netherlands"),
        "no": _("Norway"),
        "pl": _("Poland"),
        "ro": _("Romania"),
        "sc": _("Seychelles"),
        "se": _("Sweden"),
        "sg": _("Singapore"),
        "su": _("Russia"),
        "ua": _("Ukraine"),
        "uk": _("United Kingdom"),
        "us": _("United States"),
    }

    country_ids = list(nodes.keys())
    country_names = list(nodes.values())

    node = config.dconf.get_string("exit-node")
    if node == "ww":
        index = 0
    else:
        country_name = nodes[node]
        index = sorted(country_names).index(country_name) + 1

    # Define and populate the model
    model = Gio.ListStore(item_type=Country)
    model.append(Country(country_id="ww", country_name=_("Auto (Best)")))
    for country_name in sorted(country_names):
        country_id = country_ids[country_names.index(country_name)]
        model.append(Country(country_id=country_id, country_name=country_name))

    combo.set_model(model)

    combo.set_selected(index)


def on_actionacceptconnection_realize(switch):
    """
    bind accept-connection
    """
    config.dconf.bind(
        "accept-connection",
        switch,
        "active",
        Gio.SettingsBindFlags.DEFAULT,
    )


def on_actionsocksport_realize(spin):
    """
    bind socks-port
    """
    port = config.dconf.get_int("socks-port")
    spin.set_text(str(port))
    config.dconf.bind(
        "socks-port", spin, "value", Gio.SettingsBindFlags.DEFAULT
    )


def on_actiondnsport_realize(spin):
    """
    bind socks-port
    """
    port = config.dconf.get_int("dns-port")
    spin.set_text(str(port))
    config.dconf.bind("dns-port", spin, "value", Gio.SettingsBindFlags.DEFAULT)


def on_actionhttpport_realize(spin):
    """
    bind http-port
    """
    port = config.dconf.get_int("http-port")
    spin.set_text(str(port))
    config.dconf.bind(
        "http-port", spin, "value", Gio.SettingsBindFlags.DEFAULT
    )


def on_bt_factory_setup(factory, list_item):
    """
    set labels for dropdown
    """
    del factory
    label = Gtk.Label()
    list_item.set_child(label)


def on_bt_factory_bind(factory, list_item):
    """
    set bridgetype names as labels
    """
    del factory
    label = list_item.get_child()
    bridge_type = list_item.get_item()
    label.set_text(bridge_type.type_name)


def on_bridgetype_change(combo, _):
    """
    set bridge type in dconf
    """
    bridgetype = combo.get_selected_item()
    type_id = bridgetype.type_id
    config.dconf.set_int("bridge-type", type_id)
    actions.set_pluginrow_sensivity()


def on_bridgetypecombo_realize(combo):
    """
    create and set model for bridge type
    """
    bridgetype = config.dconf.get_int("bridge-type")
    _ = config._
    # Define and populate the model
    model = Gio.ListStore(item_type=BridgeType)
    model.append(BridgeType(type_id=0, type_name=_("None")))
    model.append(BridgeType(type_id=1, type_name=_("Vanilla")))
    model.append(BridgeType(type_id=2, type_name=_("Obfuscated")))
    combo.set_model(model)
    combo.set_selected(bridgetype)


def on_pluginrow_realize(row):
    """
    set row sensitive if a plugable transport is set
    """
    actions.set_pluginrow_sensivity(row)


def on_bridgetextview_realize(view):
    """
    show bridges list
    """
    buff = view.get_buffer()
    bridges_file = tasks.get_bridges_file()
    with open(bridges_file, encoding="utf-8") as file:
        text = file.read()
        buff.set_text(str(text))


def on_savebutton_clicked(*_, **__):  # TODO: Move this to actions
    """
    save bridges
    """
    actions.on_save()


def on_dialog_realize(dialog):
    """
    Add OK button to dialog
    """
    dialog.add_response("ok", config._("_OK"))
    #  These two lines are here becuase Cambalache has no support of
    #  translation for AdwMessageDialog yet. To be removed afterward
    dialog.set_heading(config._("Can not save the bridges"))
    dialog.set_body(config._("Please check the syntax of bridges"))


def on_pluginbutton_realize(button):
    """
    setup plugin button
    """
    actions.setup_pluginbutton(button)


def on_pluginbutton_clicked(button):
    """
    open file chooser
    """
    button.chooser.open(callback=on_pluginchooser_response)


def on_pluginchooser_response(chooser, task):
    """
    get plugin file and update plugin button
    """
    try:
        file = chooser.open_finish(task)
    except GLib.GError:
        # gtk-dialog-error-quark: Dismissed by user
        pass
    else:
        config.dconf.set_string("plugable-transport", file.get_path())
        actions.setup_pluginbutton()


def on_about_realize(window):
    """
    insert translators
    """
    window.set_translator_credits(config._("translator-credits"))



#!/usr/bin/python3
# Released under GPLv3+ License
# Danial Behzadi<dani.behzi@ubuntu.com>, 2020-2022.

"""
handle ui related stuff
"""

import gi

gi.require_versions({"Gtk": "4.0"})
from gi.repository import Gdk, Gtk

from . import config
from . import handler

builder = Gtk.Builder(scope_object_or_map=handler)
ui_dir = config.s_data_dir + "/ui"


def initialize_builder() -> None:
    """
    connect builder to files and handlers
    """
    builder.add_from_file(ui_dir + "/about.ui")
    builder.add_from_file(ui_dir + "/preferences.ui")
    builder.add_from_file(ui_dir + "/main.ui")


def get(obj: str):
    """
    get object from ui
    """
    return builder.get_object(obj)


def css() -> None:
    """
    apply css to ui
    """
    css_provider = Gtk.CssProvider()
    css_provider.load_from_path(ui_dir + "/style.css")
    display = Gdk.Display.get_default()
    Gtk.StyleContext.add_provider_for_display(
        display, css_provider, Gtk.STYLE_PROVIDER_PRIORITY_USER
    )



#!/usr/bin/python3
# Released under GPLv3+ License
# Danial Behzadi<dani.behzi@ubuntu.com>, 2020-2022.


"""
main configurations for carburetor
"""

import ctypes
import gettext
import locale
import os
import sys

import gi

gi.require_versions({"Adw": "1"})
from gi.repository import Gio, GLib, Adw


dconf = Gio.Settings.new("org.tractor")
s_data_dir = os.path.dirname(os.path.abspath(__file__))
locale_dir = s_data_dir + "/locales"
locale.setlocale(locale.LC_ALL, "")
if hasattr(locale, "bindtextdomain"):
    libintl = locale
elif os.name == "nt":
    libintl = ctypes.cdll.LoadLibrary("libintl-8.dll")
elif sys.platform == "darwin":
    libintl = ctypes.cdll.LoadLibrary("libintl.dylib")
libintl.bindtextdomain("carburetor", locale_dir)
gettext.bindtextdomain("carburetor", locale_dir)
libintl.textdomain("carburetor")
gettext.textdomain("carburetor")
_ = gettext.gettext
COMMAND = "tractor"
app_name = _("Carburetor")

GLib.set_application_name(app_name)
Adw.init()
